use clap::{CommandFactory, Parser, Subcommand};

mod derive;
mod mnemonic;
mod provision;
mod recover;
mod shard;
mod wizard;

/// The Kitchen Sink of Entropy.
#[derive(Parser, Clone, Debug)]
#[command(author, version, about, long_about = None)]
pub struct Keyfork {
    // Global options
    #[command(subcommand)]
    pub command: KeyforkCommands,
}

#[derive(Subcommand, Clone, Debug)]
pub enum KeyforkCommands {
    /// Derive keys of various formats. These commands require that the Keyfork server is running,
    /// which can be started by running a `keyfork recover` command.
    Derive(derive::Derive),

    /// Mnemonic generation and persistence utilities.
    Mnemonic(mnemonic::Mnemonic),

    /// Splitting and combining secrets, using Shamir's Secret Sharing.
    Shard(shard::Shard),

    /// Derive and deploy keys to hardware.
    #[command(subcommand_negates_reqs(true))]
    Provision(provision::Provision),

    /// Recover a seed using the requested recovery mechanism and start the Keyfork server.
    Recover(recover::Recover),

    /// Utilities to automatically manage the setup of Keyfork.
    Wizard(wizard::Wizard),

    /// Print an autocompletion file to standard output.
    #[cfg(feature = "completion")]
    Completion {
        #[arg(value_enum)]
        shell: clap_complete::Shell,
    },
}

impl KeyforkCommands {
    pub fn handle(&self, keyfork: &Keyfork) -> Result<(), Box<dyn std::error::Error>> {
        match self {
            KeyforkCommands::Derive(d) => {
                d.handle(keyfork)?;
            }
            KeyforkCommands::Mnemonic(m) => {
                let response = m.command.handle(m, keyfork)?;
                println!("{response}");
            }
            KeyforkCommands::Shard(s) => {
                s.command.handle(s, keyfork)?;
            }
            KeyforkCommands::Provision(p) => {
                p.handle(keyfork)?;
            }
            KeyforkCommands::Recover(r) => {
                r.handle(keyfork)?;
            }
            KeyforkCommands::Wizard(w) => {
                w.handle(keyfork)?;
            }
            #[cfg(feature = "completion")]
            KeyforkCommands::Completion { shell } => {
                let mut command = Keyfork::command();
                let command_name = command.get_name().to_string();
                clap_complete::generate(
                    *shell,
                    &mut command,
                    command_name,
                    &mut std::io::stdout(),
                );
            }
        }
        Ok(())
    }
}
